<?php

namespace Drupal\layout_settings\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Layout\LayoutInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\Core\Plugin\PluginWithFormsInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class LayoutDefaultSettings.
 */
class LayoutDefaultSettings extends ConfigFormBase {

  /**
   * Drupal\layout_settings\Plugin\LayoutSettingsManager definition.
   *
   * @var \Drupal\layout_settings\Plugin\LayoutSettingsManager
   */
  protected $pluginManagerLayoutSettings;

  /**
   * Drupal\domain_config_ui\Config\ConfigFactory definition.
   *
   * @var \Drupal\domain_config_ui\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The layout manager.
   *
   * @var \Drupal\Core\Layout\LayoutPluginManagerInterface
   */
  protected $layoutCoreManager;

  /**
   * The plugin form manager.
   *
   * @var \Drupal\Core\Plugin\PluginFormFactoryInterface
   */
  protected $pluginFormFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->pluginManagerLayoutSettings = $container->get('plugin.manager.layout_settings');
    $instance->configFactory = $container->get('config.factory');
    $instance->layoutCoreManager = $container->get('plugin.manager.core.layout');
    $instance->pluginFormFactory = $container->get('plugin_form.factory');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'layout_default_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('layout_settings.layout_default_settings')
      ->get('defaults');
    $layouts = $this->config('layout_settings.layout_default_settings')
      ->get('layouts');

    $options = [];
    $definitions = $this->layoutCoreManager->getDefinitions();
    foreach ($definitions as $id => $definition) {
      $options[$id] = $definition->getLabel();
    }
    $form['layouts'] = [
      '#type' => 'checkboxes',
      '#options' => $options,
      '#title' => $this->t('Layouts availables'),
      '#descriptions' => $this->t('Check the layouts that want to set default settings and after save they will be available.'),
      '#default_value' => $layouts,
    ];

    $form['defaults'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];
    /** @var  \Drupal\Core\Layout\LayoutDefinition $definition */
    foreach (array_filter($layouts) as $id) {
      if (empty($definitions[$id])) {
        continue;
      }
      $definition = $definitions[$id];
      $form['defaults'][$id] = [
        '#type' => 'details',
        '#title' => $definition->getLabel(),
        '#tree' => TRUE,
        '#open' => FALSE,
        'layout_settings' => [
          '#type' => 'container',
          '#states_selector' => [
            'defaults',
            '[' . $id . ']',
            '[layout_settings]',
          ],
        ],
      ];
      $default = !empty($config[$id]['layout_settings']) ? $config[$id]['layout_settings'] : [];
      $layout = $this->layoutCoreManager->createInstance($id, $default);
      $subform_state = new FormState();
      $form['defaults'][$id]['layout_settings'] = $this->getPluginForm($layout)
        ->buildConfigurationForm($form['defaults'][$id]['layout_settings'], $subform_state);
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * Retrieves the plugin form for a given layout.
   *
   * @param \Drupal\Core\Layout\LayoutInterface $layout
   *   The layout plugin.
   *
   * @return \Drupal\Core\Plugin\PluginFormInterface
   *   The plugin form for the layout.
   */
  protected function getPluginForm(LayoutInterface $layout) {
    if ($layout instanceof PluginWithFormsInterface) {
      return $this->pluginFormFactory->createInstance($layout, 'configure');
    }

    if ($layout instanceof PluginFormInterface) {
      return $layout;
    }

    throw new \InvalidArgumentException(sprintf('The "%s" layout does not provide a configuration form', $layout->getPluginId()));
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $this->config('layout_settings.layout_default_settings')
      ->set('defaults', $form_state->getValue('defaults'))
      ->set('layouts', $form_state->getValue('layouts'))
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'layout_settings.layout_default_settings',
    ];
  }

}
