<?php

namespace Drupal\layout_settings\Plugin\LayoutSettings;

use Drupal\Core\Config\Config;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\layout_settings\Plugin\LayoutSettingsBase;
use Drupal\layout_settings\Plugin\LayoutSettingsManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * @LayoutSettings(
 *  id = "predefined_classes",
 *  label = @Translation("Predefined Classes"),
 *  context = {
 *    "layout",
 *    "rows",
 *    "regions"
 *  }
 * )
 */
class PredefinedClasses extends LayoutSettingsBase {

  use StringTranslationTrait;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The config object for 'layout_settings.predefined_classes'.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $predefinedClasses;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LayoutSettingsManager $layout_settings_manager, Request $request, RendererInterface $renderer, Config $predefined_classes) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $layout_settings_manager);
    $this->request = $request;
    $this->renderer = $renderer;
    $this->predefinedClasses = $predefined_classes;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.layout_settings'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('renderer'),
      $container->get('config.factory')
        ->get('layout_settings.predefined_classes')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build(&$build, $context) {
    $region = $context[1];
    switch ($context[0]) {
      case 'layout':
        if ($region === 'wrapper') {
          foreach ($this->configuration['predefined_classes'] as $class) {
            $build['#parent_attributes']['class'][] = $class;
          }
        }
        else {
          foreach ($this->configuration['predefined_classes'] as $class) {
            $build['#attributes']['class'][] = $class;
          }
        }
        break;

      case 'rows':
        break;

      case 'regions':
        foreach ($this->configuration['predefined_classes'] as $class) {
          $build[$region]['#attributes']['class'][] = $class;
        }
        break;

      default:
        break;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultValues() {
    return self::defaultSettings() + ['predefined_classes' => []];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array &$form, FormStateInterface $form_state) {
    $element = [];
    $data = $this->configuration;
    $predefined = $this->predefinedClasses->get('predefined_classes');
    $classes = preg_split('/\n|\r\n?/', $predefined);
    $default = [];
    $options = [];
    if (!empty($classes)) {
      foreach ($classes as $class) {
        if (strlen($class) === 0) {
          continue;
        }
        $class_expl = explode('|', $class);
        if (count($class_expl) < 2) {
          continue;
        }
        $class_name = $class_expl[0];
        $label = $class_expl[1];
        $options[$class_name] = $label;
      }
    }
    $element['predefined_classes'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Classes'),
      '#options' => $options,
      '#default_value' => !empty($data['predefined_classes']) ? $data['predefined_classes'] : $default,
    ];

    return $element;
  }

}
